VSVersionInfo(
  ffi=FixedFileInfo(
    filevers=(1, 4, 0, 0),
    prodvers=(1, 4, 0, 0),
    mask=0x3f,
    flags=0x0,
    OS=0x40004,
    fileType=0x1,
    subtype=0x0,
    date=(0, 0)
    ),
  kids=[
    StringFileInfo(
      [
    StringTable(
        u'040904B0',
        [StringStruct(u'CompanyName', u'PyRGG Development Team'),
        StringStruct(u'FileDescription', u'PYRGG.exe'),
        StringStruct(u'FileVersion', u'1.4.0.0'),
        StringStruct(u'InternalName', u'PYRGG.exe'),
        StringStruct(u'LegalCopyright', u'Copyright (c) 2023 PyRGG Development Team'),
        StringStruct(u'OriginalFilename', u'PYRGG.exe'),
        StringStruct(u'ProductName', u'PYRGG'),
        StringStruct(u'ProductVersion', u'1, 4, 0, 0')])
    ]), 
    VarFileInfo([VarStruct(u'Translation', [1033, 1200])])
  ]
)