python -m autopep8 pyrgg --recursive --aggressive --aggressive --in-place --pep8-passes 2000 --verbose
python -m autopep8 setup.py --recursive --aggressive --aggressive --in-place --pep8-passes 2000 --verbose
python -m autopep8 otherfile/version_check.py --recursive --aggressive --aggressive --in-place --pep8-passes 2000 --verbose